# Functions that build a data package using GO data
#

GOPkgBuilder <- function(pkgName = "GO", pkgPath, version = "1.6.2",
                         go = GO(srcUrl = getSrcUrl("GO", xml = TRUE),
                         fromWeb = TRUE), ll = LL(srcUrl = getSrcUrl("ll"),
                         parser = file.path(.path.package("AnnBuilder"),
                         "scripts", "llNGOParser"), fromWeb = TRUE),
                         author = list(author = "anonymous",
                         maintainer = "anonymous@email.com"),
                         repList = getRepList("go", list(ll = ll, go =
                         go))){
    print("It may take me a while to process the data. Be patient!")
    if(missing(pkgPath)){
        print(paste("No path specified. The resulting package will be",
                    "stored in", getwd()))
        pkgPath <- getwd()
    }

    makeSrcInfo()
    createEmptyDPkg(pkgName, pkgPath, force = TRUE)

    # A matrix to keep GO ids and group (CC, BP, MF) information
#    goNGroup <- cbind(getTopGOid("GO"), "GO")
    #go <- GO(goUrl, fromWeb = fromWeb)
    goData <- appendObsolete(readData(go, xml = TRUE))
    tempName <- colnames(goData)
    # Add a category column
    goData <- cbind(goData, rep(NA, nrow(goData)))
    colnames(goData) <- c(tempName, "category")
    goData[goData[,1] == "GO:0003673", ncol(goData)] <- "GO root"
    # Process data for MF, CC, and BP and write the data to parents
    # and ontology term data file
    goGroups <- getGOGroupIDs()
    for(i in names(goGroups)){
        goids <- c(as.vector(goGroups[i]),
                   getOffspringNodes(goGroups[i], goData))
        # Map GO ids to the three ontology categories. This file will
        # be used later to create the go category environment
        # goNGroup <- rbind(goNGroup, cbind(goids, i))
        goData[is.element(goData[,"goid"], goids), "category"] <- i
        temp <- goData[is.element(goData[,"goid"], goids),]
        # Write GO id to parent GO id data file
        saveMat(temp[, c("goid", "association")], pkgName = pkgName,
                pkgPath = pkgPath, envName = paste(i, "PARENTS", sep = ""),
                fun = twoStepSplit)
        # Write Ontology file
        # temp <- goData[is.element(goData[,1], c(getTopGOid("GO"), goids)),]
        #saveMat(temp[, c(1, 3)], pkgName = pkgName, pkgPath = pkgPath,
        #            envName = paste(i, "ID2TERM", sep = ""),
        #            fun = splitEntry)
        # Write GO id to children GO id data file
        temp <- sapply(goids, getChildNodes, goData = goData)
        temp <- sapply(temp, paste, sep = "", collapse = ";")
        # Write GO id to parent GO id data file
        saveMat(cbind(names(temp), temp), pkgName = pkgName,
                pkgPath = pkgPath, envName = paste(i, "CHILDREN", sep = ""),
                fun = splitEntry)
        temp <- sapply(goids, getAncestors, goData = goData, keepTree = FALSE)
        saveList(temp, pkgName = pkgName, pkgPath = pkgPath,
                 envName = paste(i, "ANCESTOR", sep = ""))

        temp <- sapply(goids, getOffspringNodes, goData = goData,
                       keepTree = FALSE)
        saveList(temp, pkgName = pkgName, pkgPath = pkgPath,
                 envName = paste(i, "OFFSPRING", sep = ""))
    }
    # Write the mapping between GO ids and category information
    #ll <- LL(srcUrl = llUrl, parser = file.path(.path.package("AnnBuilder"),
    #                               "scripts", "llNGOParser"))
    llMapping <- try(parseData(ll, ncol = 2))
    # Do LL2GO ids
    goList <- getList4GO(getNamedCat(goData),
                         sapply(llMapping[, 2], twoStepSplit))
    names(goList) <- llMapping[, 1]
    saveList(goList, pkgName, pkgPath, "LOCUSID2GO")
    #saveMat(llMapping, pkgName = pkgName, fun = twoStepSplit,
    #             pkgPath = pkgPath, envName = "LL2GO")
    # Do GO2LL
    go2LL <- reverseMap4GO(llMapping, sep = ";", "ll2GO")
    tempFun <- function(x) return(twoStepSplit(x, asNumeric = TRUE))
    saveMat(go2LL, pkgName = pkgName, pkgPath = pkgPath,
                                   envName = "LOCUSID", fun = tempFun)
    go2All <- mapGO2AllProbe(go2LL, goData, "", sep = ";", all = TRUE)
    saveMat(as.matrix(cbind(names(go2All), go2All)), pkgName = pkgName,
            pkgPath = pkgPath, envName = "ALLLOCUSID", fun = tempFun)
    go2All <- go2All[!is.na(go2All)]
    ll2AllGO <- reverseMap4GO(cbind(names(go2All), go2All), ";", "GO2LL")
    goList <- getList4GO(getNamedCat(goData), sapply(ll2AllGO[,2],
                                                     twoStepSplit))
    names(goList) <- ll2AllGO[, 1]
    saveList(goList, pkgName, pkgPath, "LOCUSID2ALLGO")
    #Write the mapping between GO ids and category information
    #saveMat(goData[, c(1, ncol(goData))], pkgName = pkgName,
    #            pkgPath = pkgPath, envName = "CATEGORY")
    # saveMat(cbind(goData[, "go"], paste(goData[,"definition"],
    #                                    goData[,7], sep = "@")),
    #        fun = twoStepSplit, pkgName = pkgName,
    #        pkgPath = pkgPath, envName = "DEFINITION")
    #saveMat(cbind(goData[, "GO"], paste(goData[, "synonym"], goData[,7],
    #                                    sep = "@")), fun = twoStepSplit,
    #        pkgName = pkgName, pkgPath = pkgPath,
    #        envName = "synonym", fun = splitEntry)
    #synId2GO <- mergeRowByKey(goData[, c("synonymid", "go")])
    #synId2GO <- snyId2GO[!is.na(snyId2GO)]
    #saveMat(synId2GO, pkgName = pkgName, pkgPath = pkgPath,
    #        envName = "SECONDARY", fun = splitEntry)
    #Write the mappings between GO ids and GO terms
    #saveMat(cbind(goData[,1], paste(goData[,3], goData[,4], sep = "@")),
    #        fun = twoStepSplit, pkgName = pkgName, pkgPath = pkgPath,
    #        envName = "TERM")
    ### Column order should match the args for GOTerms constructor
    goTerms <- apply(goData[, c("goid", "ontology", "category",
                                "synonym", "synonymid", "definition")],
                     1, getGOTermsList, objOK = TRUE)
    names(goTerms) <- goData[, "goid"]
    saveList(goTerms, pkgName, pkgPath, "TERM")
    repList[["PKGNAME"]] <- "GO"
    writeDocs("", pkgName, pkgPath, version, author, repList, "GO")
    writeDatalist(pkgName, pkgPath)
    #copySubstitute(file.path(.path.package("AnnBuilder"), "scripts",
    #               "GOTerms.R"), file.path(pkgPath,
    #               pkgName, "R", "GOTerms.R"), list(QQQQQQ = ""))
    #copySubstitute(file.path(.path.package("AnnBuilder"), "templates",
    #               "GOTerms-class.Rd"), file.path(pkgPath, pkgName,
    #               "man", "GOTerms-class.Rd"), list(QQQQQQ = ""))
    #write("require(\"annotate\") || stop(\" not available\")",
    #      file.path(pkgPath, pkgName, "install.R"))
    #write("require(\"methods\") || stop(\"methods not available\")",
    #      file.path(pkgPath, pkgName, "R_PROFILE.R"))
}

writeDocs <- function(baseName, pkgName, pkgPath, version, author,
                      repList, pattern, isFile = TRUE){
    # Write man pages and other files
    writeDescription(pkgName, pkgPath, version, author)
    copyTemplates(repList, pattern, pkgName, pkgPath)
    copySubstitute(file.path(.path.package("AnnBuilder"), "templates",
                             "GENERAL.Rd"), file.path(pkgPath, pkgName,
                             "man", paste(pkgName, ".Rd", sep = "")),
                              list(PKGNAME = pkgName, SOURCENBUILT =
                                   paste("\n\t", repList, sep = "",
                                         collapse = "\n")), "#")
    writeZZZ(pkgPath, pkgName)
    writeFun(pkgPath, pkgName)
    getDPStats(baseName, pkgName, pkgPath, isFile)
    copySubstitute(file.path(.path.package("AnnBuilder"),
                        "templates", "PKGNAMEQC.Rd"),
                        file.path(pkgPath, pkgName, "man",
                        paste(pkgName, "QC.Rd", sep = "")),
                        list(PKGNAME = pkgName), "#")
    writeDatalist(pkgName, pkgPath)
    #writeMan4QC(pkgName, pkgPath)
    #write("require(\"methods\") || stop(\"methods not available\")",
    #      file.path(pkgPath, pkgName, "install.R"))
    #write("require(\"methods\") || stop(\"methods not available\")",
    #      file.path(pkgPath, pkgName, "R_PROFILE.R"))
}


copyTemplates <- function(repList, pattern, pkgName, pkgPath,
                                              replaceBy = NULL){
    templates <- gsub(pkgName, pattern, gsub("\\.rda$", ".Rd",
                     list.files(file.path(pkgPath, pkgName, "data"))))

    for(i in templates){
        if(is.null(replaceBy)){
            rdName <- file.path(pkgPath, pkgName, "man",
                            gsub(pattern, pkgName, i))
        }else{
            rdName <- file.path(pkgPath, pkgName, "man",
                            gsub(pattern, replaceBy, i))
        }
        options(show.error.messages = FALSE)
        copied <- try(copySubstitute(file.path(.path.package("AnnBuilder"),
                                 "templates", i), rdName, repList, "#"))
        options(show.error.messages = TRUE)
        if(inherits(copied, "try-error")){
            warning(paste("Can't copy", file.path(.path.package("AnnBuilder"),
                                 "templates", i)))
        }
    }
}

appendObsolete <- function(goData){

    obsoletes <- getOffspringNodes(goData[grep("obsolete",
                                               goData[, 3]), 1], goData)
    ontology <- goData[,3]
    goData[is.element(goData[,1], obsoletes), 3] <-
        paste(goData[is.element(goData[,1], obsoletes), 3],
              "(Obsolete)")

    return(goData)
}

getGOTermsList <- function(goTerms, objOK = FALSE){
    goTerms[is.na(goTerms) | is.null(goTerms) | goTerms == ""] <- NA
    if(objOK){
        return(GOTerms(as.character(goTerms[1]), as.character(goTerms[2]),
                   as.character(splitEntry(goTerms[3])),
                   as.character(splitEntry(goTerms[4])),
                   as.character(goTerms[5]),
                   as.character(goTerms[6])))
    }else{
        return(list(GOID = as.character(goTerms[1]),
                    Term = as.character(goTerms[2]),
                    Ontology = as.character(splitEntry(goTerms[3])),
                    Synonym = as.character(splitEntry(goTerms[4])),
                    Secondary = as.character(goTerms[5]),
                    Definition = as.character(goTerms[6])))
    }
}


