mapLLNGB <- function(organism, pkgName, pkgPath,
                     ugUrl = getSrcUrl("ug", organism),
                     llUrl = getSrcUrl("ll"), fromWeb = TRUE){

    repList <- getRepList4Perl(organism, ugUrl, llUrl, fromWeb)
    llNGB <- getLLNGBMap(repList, "ll2gb")
    gbNLL <- getLLNGBMap(repList, "gb2ll")
    saveMat(llNGB, fun = splitEntry, pkgName = pkgName,
                pkgPath = pkgPath, envName = "LL2ACCNUM")
    saveMat(gbNLL, fun = function(x) splitEntry(x, asNumeric = TRUE),
            pkgName = pkgName, pkgPath = pkgPath, envName = "ACCNUM2LL")

    if(fromWeb){
        unlink(c(repList[["UGFILE"]], repList[["LLFILE"]]))
    }
}

mapUGNGB <- function(organism, pkgName, pkgPath,
                     ugUrl = getSrcUrl("ug", organism),
                     llUrl = getSrcUrl("ll"), fromWeb = TRUE){

    repList <- getRepList4Perl(organism, ugUrl, llUrl, fromWeb)
    ugNGB <- getLLNGBMap(repList, "ug2gb")
    gbNUG <- getLLNGBMap(repList, "gb2ug")
    saveMat(ugNGB, fun = splitEntry, pkgName = pkgName,
                pkgPath = pkgPath, envName = "UG2ACCNUM")
    saveMat(gbNUG, fun = splitEntry,
            pkgName = pkgName, pkgPath = pkgPath, envName = "ACCNUM2UG")

    if(fromWeb){
        unlink(c(repList[["UGFILE"]], repList[["LLFILE"]]))
    }
}


getRepList4Perl <- function(organism, ugUrl = getSrcUrl("ug", organism),
                     llUrl = getSrcUrl("ll"), fromWeb = TRUE){

    repList <- list()
    if(fromWeb){
        tempUG <- loadFromUrl(ugUrl)
        tempLL <- loadFromUrl(llUrl)
    }else{
        tempUG <- ugUrl
        tempLL <- llUrl
    }
    if(.Platform$OS.type == "unix"){
        perlBin <- system("which perl", intern = TRUE)
        if(length(perlBin) > 1){
            stop("Perl is not available!")
        }
        repList[["PERLLOC"]] <- paste("#!", perlBin, sep = "")
    }else if(OS == "windows"){
        repList[["PERLLOC"]] <- "#!/usr/bin/perl -w"
    }
    repList[["UGFILE"]] <- tempUG
    repList[["LLFILE"]] <- tempLL
    #repList[["ORGANISM"]] <- getOrgName(organism, "scientific")
    repList[["ORGANISM"]] <- organism

    return(repList)
}

# Does two way mappings between LocusLink ids and GenBank Accession
# numbers. Source file used is from UniGene
# (ftp://ftp.ncbi.nih.gov/repository/UniGene).
# ug - a UG object with correct url, parser, ... set.

getLLNGBMap <- function(repList, what = "ll2gb"){
    tempPerl <- file.path(.path.package("AnnBuilder"), "temp",
                          paste(basename(tempfile("ugNllParser")),
                                ".pl", sep = ""))
    repList[["OUTFILE"]] <- file.path(.path.package("AnnBuilder"), "temp",
                          basename(tempfile("llNgb")))
    switch(toupper(what),
           LL2GB = parser <- "ll2GBPerl.temp",
           GB2LL = parser <- "gb2LLPerl.temp",
           UG2GB = parser <- "ug2GBPerl.temp",
           GB2UG = parser <- "gb2UGPerl.temp",
           stop("Unknown mapping type"))

    copySubstitute(file.path(.path.package("AnnBuilder"),
                            "templates", parser), tempPerl, repList, "#")
    .callPerl(tempPerl, .Platform$OS.type)

    parsed <- matrix(scan(repList[["OUTFILE"]], what = "character",
                           sep = "\t", quote = "", quiet = TRUE,
                           strip.white = TRUE, comment.char = ""),
                           ncol = 2, byrow = TRUE)
    unlink(c(tempPerl, repList[["OUTFILE"]]))
    return(parsed)
}
